@@ -38,6 +38,10 @@ def resolve(spec, modules):
         # Set the base name of the module.  This is done for efficiency.
         mod.py_name = mod.fq_py_name.name.split('.')[-1]
 
+    # There is nothing else that needs doing for composite modules.
+    if spec.is_composite:
+        return
+
     # Set the default meta-type for the main module if it doesn't have one
     # explicitly set.
     if spec.module.default_metatype is None:
@@ -196,8 +200,9 @@ def _resolve_module(spec, mod, error_log, final_checks, seen=None):
     for imported_mod in mod.imports:
         _resolve_module(spec, imported_mod, error_log, final_checks, seen=seen)
 
-    # Resolve typedefs, variables and global functions.
+    # Resolve typedefs, enums, variables and global functions.
     _resolve_typedefs(spec, mod, error_log)
+    _resolve_enums(spec, error_log)
     _resolve_variables(spec, mod, error_log)
     _resolve_scope_overloads(spec, mod.overloads, error_log, final_checks)
 
@@ -920,7 +925,6 @@ def _resolve_scope_overloads(spec, overloads, error_log, final_checks,
                     break
 
         if isinstance(scope, WrappedClass):
-
             if scope.deprecated and not overload.deprecated :
                 overload.deprecated = scope.deprecated
 
@@ -928,6 +932,46 @@ def _resolve_scope_overloads(spec, overloads, error_log, final_checks,
                 scope.is_abstract = True
 
 
+# The supported enum base types.  Note that we use the STRING types and the
+# BYTE types because there is no opportunity to apply the /PyInt/ annotation
+# and there can be no confusion about context.  We still need the BYTE types
+# because /PyInt/ could have been specified in a typedef.
+_ENUM_BASE_TYPES = (
+    ArgumentType.STRING,
+    ArgumentType.SSTRING,
+    ArgumentType.USTRING,
+    ArgumentType.BYTE,
+    ArgumentType.SBYTE,
+    ArgumentType.UBYTE,
+    ArgumentType.SHORT,
+    ArgumentType.USHORT,
+    ArgumentType.INT,
+    ArgumentType.UINT,
+    ArgumentType.BOOL,
+)
+
+def _resolve_enums(spec, error_log):
+    """ Resolve the base types for all the enums. """
+
+    for enum in spec.enums:
+        base_type = enum.enum_base_type
+
+        if base_type is None:
+            continue
+
+        _resolve_type(spec, enum.module, enum.scope, base_type, error_log)
+
+        # The current ABI implementations only support enums no larger than an
+        # int.
+        if base_type.type not in _ENUM_BASE_TYPES or len(base_type.derefs) != 0:
+            error_log.log(f"unsupported enum base type",
+                    source_location=base_type.source_location)
+
+        # The default is int.
+        if base_type.type is ArgumentType.INT:
+            enum.enum_base_type = None
+
+
 def _resolve_variables(spec, mod, error_log):
     """ Resolve the data types for the variables of a module. """
 
@@ -1345,6 +1389,11 @@ def _resolve_py_signature_types(spec, mod, scope, overload, error_log,
                     lambda: _check_array_support(overload, local_arg_nr, scope,
                             error_log))
 
+        # If the type of the argument is a /Movable/ mapped type then add
+        # /Transfer/.
+        if arg.type is ArgumentType.MAPPED and arg.definition.movable:
+            arg.transfer = Transfer.TRANSFER
+
         if scope is not None:
             _scope_default_value(spec, scope, arg)
 
@@ -1833,6 +1882,7 @@ def _instantiate_mapped_type_template(spec, mod, mapped_type_template, type,
     proto_mapped_type = mapped_type_template.mapped_type
 
     mapped_type.handles_none = proto_mapped_type.handles_none
+    mapped_type.movable = proto_mapped_type.movable
     mapped_type.needs_user_state = proto_mapped_type.needs_user_state
     mapped_type.no_assignment_operator = proto_mapped_type.no_assignment_operator
     mapped_type.no_copy_ctor = proto_mapped_type.no_copy_ctor
@@ -1888,7 +1938,7 @@ def _search_scope(spec, scope, scoped_name, type):
 
 
 def _name_lookup(spec, mod, scoped_name, type):
-    """ Look up a name and resole the corresponding type. """
+    """ Look up a name and resolve the corresponding type. """
 
     _search_mapped_types(spec, mod, type, scoped_name)
     if type.type is not ArgumentType.NONE:
