@@ -1,29 +1,11 @@
-# Copyright (c) 2023, Riverbank Computing Limited
-# All rights reserved.
-#
-# This copy of SIP is licensed for use under the terms of the SIP License
-# Agreement.  See the file LICENSE for more details.
-#
-# This copy of SIP may also used under the terms of the GNU General Public
-# License v2 or v3 as published by the Free Software Foundation which can be
-# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-2-Clause
+
+# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>
 
 
 from dataclasses import dataclass, field
-from enum import Enum
-from typing import Any, Dict, List, Optional, Union
+from enum import auto, Enum
+from typing import Any, Optional, Union
 
 from .scoped_name import ScopedName
 
@@ -31,22 +13,20 @@ from .scoped_name import ScopedName
 class AccessSpecifier(Enum):
     """ The class access specifiers. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # Private access.
-    PRIVATE = 0x04
+    PRIVATE = auto()
 
     # Protected access.
-    PROTECTED = 0x02
+    PROTECTED = auto()
 
     # Public access.
-    PUBLIC = 0x01
+    PUBLIC = auto()
 
 
 class ArgumentType(Enum):
     """ The types of either C/C++ or Python arguments.  The numerical values of
-    these can occur in generated code so so types must always be appended and
-    old (unused) types must never be removed.
+    these can occur in generated code so types must always be appended and old
+    (unused) types must never be removed.
     """
 
     # The type hasn't been specified.
@@ -227,45 +207,38 @@ class ArgumentType(Enum):
 class ArrayArgument(Enum):
     """ The array support provided by an argument. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # /Array/ was specified.
-    ARRAY = 0
+    ARRAY = auto()
 
     # /ArraySize/ was specified.
-    ARRAY_SIZE = 1
+    ARRAY_SIZE = auto()
 
     # The argument provides no array support.
-    NONE = 2
+    NONE = auto()
 
 
 class ClassKey(Enum):
     """ The key that identifies a particular type of class. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # A class.
-    CLASS = 0
+    CLASS = auto()
 
     # A struct.
-    STRUCT = 1
+    STRUCT = auto()
 
     # A union.
-    UNION = 2
+    UNION = auto()
 
 
 class DocstringFormat(Enum):
     """ The formatting applied to the text of the docstring. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
-    # The signature is appended to the docstring.
     # Any leading spaces common to all non-blank lines in the docstring are
     # removed.
-    DEINDENTED = 1
+    DEINDENTED = auto()
 
     # The docstring is used as it is specified in the .sip file.
-    RAW = 0
+    RAW = auto()
 
 
 class DocstringSignature(Enum):
@@ -274,338 +247,318 @@ class DocstringSignature(Enum):
     docstring then it applies to all the class's ctors.
     """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # The signature is appended to the docstring.
-    APPENDED = 2
+    APPENDED = auto()
 
     # The signature is discard.
-    DISCARDED = 0
+    DISCARDED = auto()
 
     # The signature is prepended to the docstring.
-    PREPENDED = 1
+    PREPENDED = auto()
 
 
 class EnumBaseType(Enum):
     """ The different base types fo an enum. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # enum.Enum
-    ENUM = 0
+    ENUM = auto()
 
     # enum.Flag
-    FLAG = 1
+    FLAG = auto()
 
     # enum.IntEnum
-    INT_ENUM = 2
+    INT_ENUM = auto()
 
     # enum.IntFlag
-    INT_FLAG = 3
+    INT_FLAG = auto()
 
     # enum.IntEnum with unsigned values.
-    UNSIGNED_INT_ENUM = 4
+    UINT_ENUM = auto()
 
 
 class GILAction(Enum):
     """ The action to take with the GIL when calling C/C++ code. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # The default action.
-    DEFAULT = 0
+    DEFAULT = auto()
 
     # Hold the GIL.
-    HOLD = 1
+    HOLD = auto()
 
     # Release the GIL.
-    RELEASE = 2
+    RELEASE = auto()
 
 
 class IfaceFileType(Enum):
     """ The type of an interface file. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
+    # A class.
+    CLASS = auto()
 
     # An %Exception.
-    EXCEPTION = 0
+    EXCEPTION = auto()
 
     # A %MappedType.
-    MAPPED_TYPE = 1
+    MAPPED_TYPE = auto()
 
     # A namespace.
-    NAMESPACE = 2
-
-    # A class.
-    CLASS = 3
+    NAMESPACE = auto()
 
 
 class KwArgs(Enum):
     """ The level of support for passing argument as keyword arguments. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # All named arguments can be passed as keyword arguments.
-    ALL = 1
+    ALL = auto()
 
     # Keyword arguments are not supported.
-    NONE = 0
+    NONE = auto()
 
     # All named optional arguments (ie. those with a default value) can be
     # passed as keyword arguments.
-    OPTIONAL = 2
+    OPTIONAL = auto()
 
 
 class PyQtMethodSpecifier(Enum):
     """ The PyQt-specific method specifier. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # A signal.
-    SIGNAL = 0x10
+    SIGNAL = auto()
 
     # A slot.
-    SLOT = 0x08
+    SLOT = auto()
 
 
 class PySlot(Enum):
     """ The Python slots corresponding to entries in a type object. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # tp_str
-    STR = 0
+    STR = auto()
 
     # tp_as_number.nb_int
-    INT = 1
+    INT = auto()
 
     # tp_as_number.nb_float
-    FLOAT = 2
+    FLOAT = auto()
 
     # tp.as_mapping.mp_length and tp.as_sequence.sq_length
-    LEN = 3
+    LEN = auto()
 
     # tp.as_sequence.sq_contains
-    CONTAINS = 4
+    CONTAINS = auto()
 
     # tp_as_number.nb_add
-    ADD = 5
+    ADD = auto()
 
     # tp.as_sequence.sq_concat
-    CONCAT = 6
+    CONCAT = auto()
 
     # tp_as_number.nb_subtract
-    SUB = 7
+    SUB = auto()
 
     # tp_as_number.nb_multiply
-    MUL = 8
+    MUL = auto()
 
     # tp.as_sequence.sq_repeat
-    REPEAT = 9
+    REPEAT = auto()
 
     # tp_as_number.nb_remainder
-    MOD = 11
+    MOD = auto()
 
     # tp_as_number.nb_floor_divide
-    FLOORDIV = 12
+    FLOORDIV = auto()
 
     # tp_as_number.nb_true_divide
-    TRUEDIV = 13
+    TRUEDIV = auto()
 
     # tp_as_number.nb_and
-    AND = 14
+    AND = auto()
 
     # tp_as_number.nb_or
-    OR = 15
+    OR = auto()
 
     # tp_as_number.nb_xor
-    XOR = 16
+    XOR = auto()
 
     # tp_as_number.nb_lshift
-    LSHIFT = 17
+    LSHIFT = auto()
 
     # tp_as_number.nb_rshift
-    RSHIFT = 18
+    RSHIFT = auto()
 
     # tp_as_number.nb_inplace_add
-    IADD = 19
+    IADD = auto()
 
     # tp.as_sequence.sq_inplace_concat
-    ICONCAT = 20
+    ICONCAT = auto()
 
     # tp_as_number.nb_inplace_subtract
-    ISUB = 21
+    ISUB = auto()
 
     # tp_as_number.nb_inplace_multiply
-    IMUL = 22
+    IMUL = auto()
 
     # tp.as_sequence.sq_inplace_repeat
-    IREPEAT = 23
+    IREPEAT = auto()
 
     # tp_as_number.nb_inplace_remainder
-    IMOD = 25
+    IMOD = auto()
 
     # tp_as_number.nb_inplace_floor_divide
-    IFLOORDIV = 26
+    IFLOORDIV = auto()
 
     # tp_as_number.nb_inplace_true_divide
-    ITRUEDIV = 27
+    ITRUEDIV = auto()
 
     # tp_as_number.nb_inplace_and
-    IAND = 28
+    IAND = auto()
 
     # tp_as_number.nb_inplace_or
-    IOR = 29
+    IOR = auto()
 
     # tp_as_number.nb_inplace_xor
-    IXOR = 30
+    IXOR = auto()
 
     # tp_as_number.nb_inplace_lshift
-    ILSHIFT = 31
+    ILSHIFT = auto()
 
     # tp_as_number.nb_inplace_rshift
-    IRSHIFT = 32
+    IRSHIFT = auto()
 
     # tp_as_number.nb_invert
-    INVERT = 33
+    INVERT = auto()
 
     # tp_call
-    CALL = 34
+    CALL = auto()
 
     # tp.as_mapping.mp_subscript and tp.as_sequence.sq_item
-    GETITEM = 35
+    GETITEM = auto()
 
     # tp.as_mapping.mp_ass_subscript and tp.as_sequence.sq_ass_item
-    SETITEM = 36
+    SETITEM = auto()
 
     # tp.as_mapping.mp_ass_subscript and tp.as_sequence.sq_ass_item
-    DELITEM = 37
+    DELITEM = auto()
 
     # tp_richcompare (Py_LT)
-    LT = 38
+    LT = auto()
 
     # tp_richcompare (Py_LE)
-    LE = 39
+    LE = auto()
 
     # tp_richcompare (Py_EQ)
-    EQ = 40
+    EQ = auto()
 
     # tp_richcompare (Py_NE)
-    NE = 41
+    NE = auto()
 
     # tp_richcompare (Py_GT)
-    GT = 42
+    GT = auto()
 
     # tp_richcompare (Py_GE)
-    GE = 43
+    GE = auto()
 
     # tp_as_number.nb_bool
-    BOOL = 45
+    BOOL = auto()
 
     # tp_as_number.nb_negative
-    NEG = 46
+    NEG = auto()
 
     # tp_as_number.nb_positive
-    POS = 47
+    POS = auto()
 
     # tp_as_number.nb_absolute
-    ABS = 48
+    ABS = auto()
 
     # tp_repr
-    REPR = 49
+    REPR = auto()
 
     # tp_hash
-    HASH = 50
+    HASH = auto()
 
     # tp_as_number.nb_index
-    INDEX = 51
+    INDEX = auto()
 
     # tp_iter
-    ITER = 52
+    ITER = auto()
 
     # tp_iter_next
-    NEXT = 53
+    NEXT = auto()
 
     # tp_setattr
-    SETATTR = 54
+    SETATTR = auto()
 
     # Internal to the parser (implemented as tp_setattr)
-    DELATTR = 55
+    DELATTR = auto()
 
     # tp_as_number.nb_matrix_multiply
-    MATMUL = 56
+    MATMUL = auto()
 
     # tp_as_number.nb_inplace_matrix_multiply
-    IMATMUL = 57
+    IMATMUL = auto()
 
     # tp_as_async.am_await
-    AWAIT = 58
+    AWAIT = auto()
 
     # tp_as_async.am_aiter
-    AITER = 59
+    AITER = auto()
 
     # tp_as_async.am_anext
-    ANEXT = 60
+    ANEXT = auto()
 
 
 class QualifierType(Enum):
     """ The type of a qualifier used in %If/%End directives. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # The qualifier is a feature.
-    FEATURE = 2
+    FEATURE = auto()
 
     # The qualifier is a platform.
-    PLATFORM = 1
+    PLATFORM = auto()
 
     # The qualifier is part of a timeline.
-    TIME = 0
+    TIME = auto()
 
 
 class Transfer(Enum):
     """ The different types of ownership transfer. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # No transfer of ownership.
-    NONE = 0
+    NONE = auto()
 
     # /Transfer/ was specified.
-    TRANSFER = 1
+    TRANSFER = auto()
 
     # /TransferBack/ was specified.
-    TRANSFER_BACK = 2
+    TRANSFER_BACK = auto()
 
     # /TransferThis/ was specified.
-    TRANSFER_THIS = 3
+    TRANSFER_THIS = auto()
 
 
 class ValueType(Enum):
     """ The different types of a value in an expression. """
 
-    # TODO: Change the values to auto() once the C code has been replaced.
-
     # A quoted character.
-    QCHAR = 0
+    QCHAR = auto()
 
     # A string.
-    STRING = 1
+    STRING = auto()
 
     # A number.
-    NUMERIC = 2
+    NUMERIC = auto()
 
     # A floating point number.
-    REAL = 3
+    REAL = auto()
 
     # A scoped name.
-    SCOPED = 4
+    SCOPED = auto()
 
     # A function call.
-    FCALL = 5
+    FCALL = auto()
 
     # A placeholder.
-    EMPTY = 6
+    EMPTY = auto()
 
 
 @dataclass
@@ -622,14 +575,14 @@ class Argument:
     array: ArrayArgument = ArrayArgument.NONE
 
     # The optional default value.
-    default_value: Optional[List['Value']] = None
+    default_value: Optional[list['Value']] = None
 
     # The optional definition.  What this is depends on the type.
     definition: Any = None
 
     # The sequence of dereferences.  An element is True if the corresponding
     # dereference is const.
-    derefs: List[bool] = field(default_factory=list)
+    derefs: list[bool] = field(default_factory=list)
 
     # Set if /DisallowNone/ was specified.
     disallow_none: bool = False
@@ -679,6 +632,11 @@ class Argument:
     # The non-default type hints.
     type_hints: Optional['TypeHints'] = None
 
+    def __hash__(self):
+        """ Reimplemented so an Argument object can be used as a dict key. """
+
+        return id(self)
+
 
 @dataclass
 class CachedName:
@@ -731,8 +689,8 @@ class Constructor:
     # The C/C++ signature.  It will be none if /NoDerived/ was specified.
     cpp_signature: Optional['Signature'] = None
 
-    # Set if /Deprecated/ was specified.
-    deprecated: bool = False
+    # deprecated message if /Deprecated/ was specified.
+    deprecated: Optional[str] = None
 
     # The docstring.
     docstring: Optional['Docstring'] = None
@@ -764,7 +722,7 @@ class Constructor:
     # Set if a Python exception is raised.
     raises_py_exception: bool = False
 
-    # The optional throw arguments.
+    # The optional throw arguments.  Replace with 'noexcept' in SIP v7.
     throw_args: Optional['ThrowArguments'] = None
 
     # Any transfer of ownership.
@@ -804,7 +762,7 @@ class FunctionCall:
     result: Argument
 
     # The list of arguments.
-    args: List[List['Value']]
+    args: list[list['Value']]
 
 
 @dataclass
@@ -838,10 +796,10 @@ class IfaceFile:
     needed: bool = False
 
     # The %TypeHeaderCode.
-    type_header_code: List[CodeBlock] = field(default_factory=list)
+    type_header_code: list[CodeBlock] = field(default_factory=list)
 
     # The interface files used by this one (either directly or indirectly).
-    used: List['IfaceFile'] = field(default_factory=list)
+    used: list['IfaceFile'] = field(default_factory=list)
 
 
 @dataclass
@@ -887,7 +845,10 @@ class MappedType:
     instance_code: Optional[CodeBlock] = None
 
     # The member functions.
-    members: List['Member'] = field(default_factory=list)
+    members: list['Member'] = field(default_factory=list)
+
+    # Set if /Movable/ was specified.
+    movable: bool = False
 
     # Set if the handwritten code requires user state information.
     needs_user_state: bool = False
@@ -905,7 +866,7 @@ class MappedType:
     no_release: bool = False
 
     # The overloaded member functions.
-    overloads: List['Overload'] = field(default_factory=list)
+    overloads: list['Overload'] = field(default_factory=list)
 
     # The Python name.  It will be None for mapped type templates.
     py_name: Optional[CachedName] = None
@@ -917,7 +878,7 @@ class MappedType:
     release_code: Optional[CodeBlock] = None
 
     # The %TypeCode.
-    type_code: List[CodeBlock] = field(default_factory=list)
+    type_code: list[CodeBlock] = field(default_factory=list)
 
     # The %TypeHintCode.
     type_hint_code: Optional[CodeBlock] = None
@@ -959,6 +920,9 @@ class Member:
     # Set if /Sequence/ was specified.
     is_sequence: bool = False
 
+    # The number of the member. (outputter)
+    member_nr: int = -1
+
     # The original interface file if the function was defined in a namespace.
     namespace_iface_file: Optional[IfaceFile] = None
 
@@ -974,13 +938,13 @@ class Module:
     """ Encapsulate a module. """
 
     # The list of all (explicit and implied) imports. (resolver)
-    all_imports: List['Module'] = field(default_factory=list)
+    all_imports: list['Module'] = field(default_factory=list)
 
     # Set if wrapped ctors should support cooperative multi-inheritance.
     call_super_init: bool = False
 
     # The text specified by any %Copying directives.
-    copying: List[CodeBlock] = field(default_factory=list)
+    copying: list[CodeBlock] = field(default_factory=list)
 
     # The default docstring format.
     default_docstring_format: DocstringFormat = DocstringFormat.RAW
@@ -1009,25 +973,25 @@ class Module:
     fq_py_name: Optional[CachedName] = None
 
     # The global functions.
-    global_functions: List[Member] = field(default_factory=list)
+    global_functions: list[Member] = field(default_factory=list)
 
     # Set if any class defined in the module has a delayed dtor.
     has_delayed_dtors: bool = False
 
     # The list of direct imports.
-    imports: List['Module'] = field(default_factory=list)
+    imports: list['Module'] = field(default_factory=list)
 
     # The code specified by any %InitialisationCode directives.
-    initialisation_code: List[CodeBlock] = field(default_factory=list)
+    initialisation_code: list[CodeBlock] = field(default_factory=list)
 
     # The software license.
     license: Optional[License] = None
 
     # The %ModuleCode.
-    module_code: List[CodeBlock] = field(default_factory=list)
+    module_code: list[CodeBlock] = field(default_factory=list)
 
     # The %ModuleHeaderCode.
-    module_header_code: List[CodeBlock] = field(default_factory=list)
+    module_header_code: list[CodeBlock] = field(default_factory=list)
 
     # The next key to auto-allocate.
     next_key: int = -1
@@ -1036,7 +1000,7 @@ class Module:
     nr_exceptions: int = 0
 
     # The generated types needed by this module. (resolver)
-    needed_types: List[Argument] = field(default_factory=list)
+    needed_types: list[Argument] = field(default_factory=list)
 
     # The number of typedefs defined in this module.
     nr_typedefs: int = 0
@@ -1045,16 +1009,16 @@ class Module:
     nr_virtual_error_handlers: int = 0
 
     # The overloaded global functions.
-    overloads: List['Overload'] = field(default_factory=list)
+    overloads: list['Overload'] = field(default_factory=list)
 
     # The code specified by any %PostInitialisationCode directives.
-    postinitialisation_code: List[CodeBlock] = field(default_factory=list)
+    postinitialisation_code: list[CodeBlock] = field(default_factory=list)
 
     # The proxy classes.
-    proxies: List['WrappedClass'] = field(default_factory=list)
+    proxies: list['WrappedClass'] = field(default_factory=list)
 
     # The code specified by any %PreInitialisationCode directives.
-    preinitialisation_code: List[CodeBlock] = field(default_factory=list)
+    preinitialisation_code: list[CodeBlock] = field(default_factory=list)
 
     # The name of the module. (resolver)
     py_name: Optional[str] = None
@@ -1063,16 +1027,16 @@ class Module:
     py_ssize_t_clean: bool = False
 
     # The list of qualifiers.
-    qualifiers: List['Qualifier'] = field(default_factory=list)
+    qualifiers: list['Qualifier'] = field(default_factory=list)
 
     # The %TypeHintCode.
-    type_hint_code: List[CodeBlock] = field(default_factory=list)
+    type_hint_code: list[CodeBlock] = field(default_factory=list)
 
     # The %UnitCode.
-    unit_code: List[CodeBlock] = field(default_factory=list)
+    unit_code: list[CodeBlock] = field(default_factory=list)
 
     # The %UnitPostIncludeCode.
-    unit_postinclude_code: List[CodeBlock] = field(default_factory=list)
+    unit_postinclude_code: list[CodeBlock] = field(default_factory=list)
 
     # Set if the actual argument names to wrapped callables should be used in
     # the generated bindings rather than automatically generated ones.
@@ -1082,7 +1046,7 @@ class Module:
     use_limited_api: bool = False
 
     # The interface files used by the module.
-    used: List[IfaceFile] = field(default_factory=list)
+    used: list[IfaceFile] = field(default_factory=list)
 
 
 @dataclass
@@ -1095,7 +1059,7 @@ class Overload:
     # The member common to all overloads.
     common: Member
 
-    # The C/C++ name.
+    # The C/C++ name if not a operator/slot.
     cpp_name: str
 
     # The C/C++ signature.
@@ -1110,9 +1074,9 @@ class Overload:
     # Set if the overload is really protected.
     access_is_really_protected: bool = False
 
-    # Set if /Deprecated/ was specified.
-    deprecated: bool = False
-
+    # deprecated message if /Deprecated/ was specified.
+    deprecated: Optional[str] = None
+    
     # The docstring.
     docstring: Optional[Docstring] = None
 
@@ -1191,7 +1155,7 @@ class Overload:
     # The source location.
     source_location: Optional['SourceLocation'] = None
 
-    # The optional throw arguments.
+    # The optional throw arguments.  Replace with 'noexcept' in SIP v7.
     throw_args: Optional['ThrowArguments'] = None
 
     # Any transfer of ownership.
@@ -1245,7 +1209,8 @@ class Qualifier:
     order: int = 0
 
     # The timeline number within the defining module if it is a TIME qualifier.
-    timeline: int = 0
+    # A negative value implies the SIP timeline.
+    timeline: int = -1
 
 
 @dataclass
@@ -1253,9 +1218,9 @@ class Signature:
     """ Encapsulate a signature (including the optional result). """
 
     # The list of arguments.
-    args: List[Argument] = field(default_factory=list)
+    args: list[Argument] = field(default_factory=list)
 
-    # The type of the optional result.
+    # The type of the result.
     result: Optional[Argument] = None
 
 
@@ -1278,7 +1243,7 @@ class Specification:
     """ Encapsulate a parsed .sip file. """
 
     # The version of the ABI being targeted.
-    abi_version: tuple
+    target_abi: Optional[tuple]
 
     # Set if the specification is strict.
     is_strict: bool
@@ -1291,62 +1256,62 @@ class Specification:
     c_bindings: bool = False
 
     # The list of classes.
-    classes: List['WrappedClass'] = field(default_factory=list)
+    classes: list['WrappedClass'] = field(default_factory=list)
 
     # The list of enums.
-    enums: List['WrappedEnum'] = field(default_factory=list)
+    enums: list['WrappedEnum'] = field(default_factory=list)
 
     # The list of exceptions.
-    exceptions: List['WrappedException'] = field(default_factory=list)
+    exceptions: list['WrappedException'] = field(default_factory=list)
 
     # The %ExportedHeaderCode.
-    exported_header_code: List[CodeBlock] = field(default_factory=list)
+    exported_header_code: list[CodeBlock] = field(default_factory=list)
 
     # The %ExportedTypeHintCode.
-    exported_type_hint_code: List[CodeBlock] = field(default_factory=list)
+    exported_type_hint_code: list[CodeBlock] = field(default_factory=list)
 
     # The extracts.
-    extracts: List[Extract] = field(default_factory=list)
+    extracts: list[Extract] = field(default_factory=list)
 
     # The interface files.
-    iface_files: List[IfaceFile] = field(default_factory=list)
+    iface_files: list[IfaceFile] = field(default_factory=list)
 
     # Set if the specification is for a composite module.
     is_composite: bool = False
 
     # The mapped type templates.
-    mapped_type_templates: List[MappedTypeTemplate] = field(default_factory=list)
+    mapped_type_templates: list[MappedTypeTemplate] = field(default_factory=list)
 
     # The mapped types.
-    mapped_types: List[MappedType] = field(default_factory=list)
+    mapped_types: list[MappedType] = field(default_factory=list)
 
     # The module for which code is to be generated.
     module: Module = field(default_factory=Module)
 
     # The cache of names that may be required as strings in the generated code.
-    name_cache: Dict[int, List[CachedName]] = field(default_factory=dict)
+    name_cache: dict[int, list[CachedName]] = field(default_factory=dict)
 
     # The number of virtual handlers. (resolver)
     nr_virtual_handlers: int = 0
 
     # The list of plugins.  Note that these are PyQt-specific and will be
     # removed in SIP v7.
-    plugins: List[str] = field(default_factory=list)
+    plugins: list[str] = field(default_factory=list)
 
     # The QObject class.
     pyqt_qobject: Optional['WrappedClass'] = None
 
     # The list of typedefs.
-    typedefs: List['WrappedTypedef'] = field(default_factory=list)
+    typedefs: list['WrappedTypedef'] = field(default_factory=list)
 
     # The list of variables.
-    variables: List['WrappedVariable'] = field(default_factory=list)
+    variables: list['WrappedVariable'] = field(default_factory=list)
 
     # The list of virtual error handlers.
-    virtual_error_handlers: List['VirtualErrorHandler'] = field(default_factory=list)
+    virtual_error_handlers: list['VirtualErrorHandler'] = field(default_factory=list)
 
     # The list of virtual handlers. (resolver)
-    virtual_handlers: List['VirtualHandler'] = field(default_factory=list)
+    virtual_handlers: list['VirtualHandler'] = field(default_factory=list)
 
     def __hash__(self):
         """ Reimplemented so a Specification object can be used as a dict key.
@@ -1371,8 +1336,8 @@ class ThrowArguments:
     """ Encapsulate the arguments to a C++ throw(). """
 
     # The list of the argument names. If it is None then 'noexcept' was
-    # specified.
-    arguments: Optional[List['WrappedException']] = None
+    # specified, otherwise there will be at least one argument.
+    arguments: Optional[list['WrappedException']] = None
 
 
 @dataclass
@@ -1504,7 +1469,7 @@ class WrappedClass:
     cannot_copy: bool = False
 
     # The list of operator casts.
-    casts: List[Argument] = field(default_factory=list)
+    casts: list[Argument] = field(default_factory=list)
 
     # The specific type of class.  It will be None for namespaces.
     class_key: Optional[ClassKey] = None
@@ -1519,10 +1484,10 @@ class WrappedClass:
     convert_to_type_code: Optional[CodeBlock] = None
 
     # The constructors.
-    ctors: List[Constructor] = field(default_factory=list)
+    ctors: list[Constructor] = field(default_factory=list)
 
     # The dtor's %PreMethodCode and %MethodCode.
-    dealloc_code: List[CodeBlock] = field(default_factory=list)
+    dealloc_code: list[CodeBlock] = field(default_factory=list)
 
     # The constructor that has /Default/ specified.
     default_ctor: Optional[Constructor] = None
@@ -1530,8 +1495,8 @@ class WrappedClass:
     # Set if /DelayDtor/ was specified.
     delay_dtor: bool = False
 
-    # Set if /Deprecated/ was specified.
-    deprecated: bool = False
+    # deprecated message if /Deprecated/ was specified.
+    deprecated: Optional[str] = None
 
     # The docstring.
     docstring: Optional[Docstring] = None
@@ -1542,7 +1507,7 @@ class WrappedClass:
     # The action required on the GIL.
     dtor_gil_action: GILAction = GILAction.DEFAULT
 
-    # The optional dtor throw arguments.
+    # The optional dtor throw arguments.  Replace with 'noexcept' in SIP v7.
     dtor_throw_args: Optional[ThrowArguments] = None
 
     # The code specified by any dtor %VirtualCatcherCode directive.
@@ -1551,6 +1516,9 @@ class WrappedClass:
     # Set if /ExportDerived/ was specified.
     export_derived: bool = False
 
+    # Set if /ExportDerivedLocally/ was specified.
+    export_derived_locally: bool = False
+
     # Set if /External/ was specified.
     external: bool = False
 
@@ -1596,8 +1564,11 @@ class WrappedClass:
     # Set if the class is QObject or a sub-class. (resolver)
     is_qobject: bool = False
 
+    # The C++ name of any overload annotated with __len__.
+    len_cpp_name: Optional[str] = None
+
     # The methods.
-    members: List[Member] = field(default_factory=list)
+    members: list[Member] = field(default_factory=list)
 
     # The value of /Metatype/ if specified.
     metatype: Optional[CachedName] = None
@@ -1607,7 +1578,7 @@ class WrappedClass:
 
     # The list of all classes in the class hierarchy starting with itself.
     # (resolver)
-    mro: List['WrappedClass'] = field(default_factory=list)
+    mro: list['WrappedClass'] = field(default_factory=list)
 
     # Set if the class needs an array helper. (resolver)
     needs_array_helper: bool = False
@@ -1629,19 +1600,19 @@ class WrappedClass:
     no_type_name: bool = False
 
     # The overloaded methods.
-    overloads: List[Overload] = field(default_factory=list)
+    overloads: list[Overload] = field(default_factory=list)
 
     # The %PickleCode.
     pickle_code: Optional[CodeBlock] = None
 
     # The properties.
-    properties: List[Property] = field(default_factory=list)
+    properties: list[Property] = field(default_factory=list)
 
     # The /PyQtFlags/.
     pyqt_flags: int = 0
 
     # The /PyQtFlagsEnums/.
-    pyqt_flags_enums: Optional[List[str]] = None
+    pyqt_flags_enums: Optional[list[str]] = None
 
     # The /PyQtInterface/.
     pyqt_interface: Optional[str] = None
@@ -1655,8 +1626,9 @@ class WrappedClass:
     # The sub-class base class. (resolver)
     subclass_base: Optional['WrappedClass'] = None
 
-    # The super-classes.
-    superclasses: List['WrappedClass'] = field(default_factory=list)
+    # The super-classes.  A super-class can only be a template argument in a
+    # class template.
+    superclasses: list[Union[Argument, 'WrappedClass']] = field(default_factory=list)
 
     # The value of /Supertype/ if specified.
     supertype: Optional[CachedName] = None
@@ -1665,7 +1637,10 @@ class WrappedClass:
     template: Optional[Template] = None
 
     # The %TypeCode.
-    type_code: List[CodeBlock] = field(default_factory=list)
+    type_code: list[CodeBlock] = field(default_factory=list)
+
+    # The %TypeDerivedCode.
+    type_derived_code: list[CodeBlock] = field(default_factory=list)
 
     # The %TypeHintCode.
     type_hint_code: Optional[CodeBlock] = None
@@ -1677,10 +1652,15 @@ class WrappedClass:
     virtual_error_handler: Optional[str] = None
 
     # The virtual overloaded methods. (resolver)
-    virtual_overloads: List[VirtualOverload] = field(default_factory=list)
+    virtual_overloads: list[VirtualOverload] = field(default_factory=list)
 
     # The visible member functions. (resolver)
-    visible_members: List[VisibleMember] = field(default_factory=list)
+    visible_members: list[VisibleMember] = field(default_factory=list)
+
+    def __hash__(self):
+        """ Reimplemented so an Argument object can be used as a dict key. """
+
+        return id(self)
 
 
 @dataclass
@@ -1696,17 +1676,20 @@ class WrappedEnum:
     # The defining module.
     module: Module
 
+    # The C++11 enum base type.
+    enum_base_type: Optional[Argument] = None
+
     # The cached fully qualified C++ name.
     cached_fq_cpp_name: Optional[CachedName] = None
 
-    # Set if the enum is defined in the protected section.
+    # Set if the enum is defined in a protected section.
     is_protected: bool = False
 
     # Set if the enum is a scoped enum.
     is_scoped: bool = False
 
     # The members.
-    members: List['WrappedEnumMember'] = field(default_factory=list)
+    members: list['WrappedEnumMember'] = field(default_factory=list)
 
     # Set if this enum is needed by the module for which code is to be
     # generated. (resolver)
@@ -1719,7 +1702,7 @@ class WrappedEnum:
     no_type_hint: bool = False
 
     # The overloaded slot member functions. (resolver)
-    overloads: List['Overload'] = field(default_factory=list)
+    overloads: list['Overload'] = field(default_factory=list)
 
     # The Python name.
     py_name: Optional[CachedName] = None
@@ -1729,11 +1712,16 @@ class WrappedEnum:
 
     # The slot member functions.  These can only be created by global operators
     # being moved. (resolver)
-    slots: List[Member] = field(default_factory=list)
+    slots: list[Member] = field(default_factory=list)
 
     # The generated type number. (resolver)
     type_nr: int = -1
 
+    def __hash__(self):
+        """ Reimplemented so an Argument object can be used as a dict key. """
+
+        return id(self)
+
 
 @dataclass
 class WrappedEnumMember:
